﻿using System;
using System.Threading.Tasks;
using Flurl;
using Flurl.Http;
using UDFUpdater.APIClient.Models;

namespace UDFUpdater.APIClient.Endpoints
{
    public class Issues
    {
        private readonly Url _baseRequest;
        private readonly string _apiKey;

        public Issues(Url baseRequest, string apiKey)
        {
            _baseRequest = baseRequest;
            _apiKey = apiKey;
        }
	
        public Task<QueryResultsContainer<ReadIssue>> List(bool shouldIncludeNotes, params int[] issueNumbers){
            return _baseRequest
                .WithIssuetrakAuthentication(_apiKey)
                .AppendPathSegments("issues", shouldIncludeNotes, "list")
                .WithHeader("X-IssueTrak-API-Request-ID", Guid.NewGuid())
                .SetQueryParam("issueNumbers",issueNumbers)
                .GetJsonAsync<QueryResultsContainer<ReadIssue>>();
        }
	
        public Task<ReadIssue> Get(bool shouldIncludeNotes, int issueNumber){
            return _baseRequest
                .WithIssuetrakAuthentication(_apiKey)
                .AppendPathSegments("issues", shouldIncludeNotes, issueNumber)
                .WithHeader("X-IssueTrak-API-Request-ID", Guid.NewGuid())
                .GetJsonAsync<ReadIssue>();
        }

        public async Task<bool> Update(UpdateIssue updateIssue)
        {
            return (await _baseRequest
                .WithIssuetrakAuthentication(_apiKey)
                .AppendPathSegments("issues")
                .WithHeader("X-IssueTrak-API-Request-ID", Guid.NewGuid())
                .PutJsonAsync(updateIssue)
                .ConfigureAwait(true))
                .IsSuccessStatusCode;
        }
    }
}