﻿using System;
using System.Globalization;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using Flurl.Http;

namespace UDFUpdater.APIClient
{
    public static class IssuetrakAuthentication
    {
        public static IFlurlRequest WithIssuetrakAuthentication(this Flurl.Url url, string apiKey)
        {
            var clone = url?.Clone() ?? throw new ArgumentNullException(nameof(url));
            return new FlurlRequest(clone)
                .WithIssuetrakAuthentication(apiKey);
        }
	
        public static T WithIssuetrakAuthentication<T>(this T client, string apiKey) where T: IHttpSettingsContainer{
            client.Settings.BeforeCallAsync = (call) => BeforeCallAsync(call, apiKey);
            return client;
        }
	
        private static async Task BeforeCallAsync(HttpCall call, string apiKey)
        {
            var requestId = call.FlurlRequest.Headers["X-IssueTrak-API-Request-ID"];
            var time = DateTime.UtcNow.ToString("O", new DateTimeFormatInfo());
            var verb = call.Request.Method.Method.ToUpper(CultureInfo.InvariantCulture);
            var path = call.Request.RequestUri.AbsolutePath.ToLowerInvariant();
            var query = call.Request.RequestUri.Query;
            var content = await (call.Request?.Content?.ReadAsStringAsync() ?? Task.FromResult(""));
            var concatenateContent = string.Join("\n",verb,requestId, time,path,query,content);
            var hash = ComputeHmacHash(apiKey, concatenateContent);
            call.FlurlRequest.Headers.Add("X-IssueTrak-API-Timestamp", time);
            call.FlurlRequest.Headers.Add("X-IssueTrak-API-Authorization", hash);
        }
	
        private static string ComputeHmacHash(string hashKey, string messageToBeHashed)
        {
            var hashKeyBytes = Encoding.UTF8.GetBytes(hashKey);
            using var hmacHashGenerator = new HMACSHA512(hashKeyBytes);
            var messageToBeHashedBytes = Encoding.UTF8.GetBytes(messageToBeHashed);
            return Convert.ToBase64String(hmacHashGenerator.ComputeHash(messageToBeHashedBytes));
		
        }
    }
}