﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace UDFUpdater.APIClient.Models
{
    public class UpdateIssue
    {
        public UpdateIssue(){}

        public UpdateIssue(ReadIssue readIssue)
            :this(readIssue, 
                readIssue?.UserDefinedFields?.Select(udf => new UpdateUserDefinedField(udf)) ?? Enumerable.Empty<UpdateUserDefinedField>())
        {
        }

        public UpdateIssue(ReadIssue readIssue, IEnumerable<UpdateUserDefinedField> userDefinedFields)
        {
            var dummy = readIssue ?? throw new ArgumentNullException(nameof(readIssue));
            IssueNumber = readIssue.IssueNumber;
            Subject = readIssue.Subject;
            Description = readIssue.Description;
            IsDescriptionRichText = readIssue.IsDescriptionRichText;
            IssueTypeId = readIssue.IssueTypeId;
            IssueSubTypeId = readIssue.IssueSubTypeId;
            IssueSubType2Id = readIssue.IssueSubType2Id;
            IssueSubType3Id = readIssue.IssueSubType3Id;
            IssueSubType4Id = readIssue.IssueSubType4Id;
            PriorityId = readIssue.PriorityId;
            AssetNumber = readIssue.AssetNumber;
            LocationId = readIssue.LocationId;
            SubmittedBy = readIssue.SubmittedBy;
            AssignedTo = readIssue.AssignedTo;
            TargetDate = readIssue.TargetDate;
            RequiredByDate = readIssue.RequiredByDate;
            NextActionTo = readIssue.NextActionTo;
            SubStatusId = readIssue.SubStatusId;
            ProjectId = readIssue.ProjectId;
            OrganizationId = readIssue.OrganizationId;
            ShouldNeverSendEmailForIssue = readIssue.ShouldNeverSendEmailForIssue;
            ClassId = readIssue.ClassId;
            DepartmentId = readIssue.DepartmentId;
            SpecialFunction1 = readIssue.SpecialFunction1;
            SpecialFunction2 = readIssue.SpecialFunction2;
            SpecialFunction3 = readIssue.SpecialFunction3;
            SpecialFunction4 = readIssue.SpecialFunction4;
            SpecialFunction5 = readIssue.SpecialFunction5;
            IssueSolution = readIssue.IssueSolution;
            Status = readIssue.Status;
            ClosedBy = readIssue.ClosedBy;
            ClosedDate = readIssue.ClosedDate;
            CauseId = readIssue.CauseId;
            SeverityId = readIssue.SeverityId;
            ResponseTime = readIssue.ResponseTime;
            IsFirstCallResolution = readIssue.IsFirstCallResolution;
            UserDefinedFields = userDefinedFields;
        }

        public int IssueNumber { get; set; }
        public string Subject { get; set; } = string.Empty;
        public string Description { get; set; } = string.Empty;
        public bool IsDescriptionRichText { get; set; }
        public int? IssueTypeId { get; set; }
        public int? IssueSubTypeId { get; set; }
        public int? IssueSubType2Id { get; set; }
        public int? IssueSubType3Id { get; set; }
        public int? IssueSubType4Id { get; set; }
        public int? PriorityId { get; set; }
        public int? AssetNumber { get; set; }
        public string? LocationId { get; set; }
        public string SubmittedBy { get; set; } = string.Empty;
        public string? AssignedTo { get; set; }
        public DateTime? TargetDate { get; set; }
        public DateTime? RequiredByDate { get; set; }
        public string? NextActionTo { get; set; }
        public int? SubStatusId { get; set; }
        public int? ProjectId { get; set; }
        public int? OrganizationId { get; set; }
        public bool? ShouldNeverSendEmailForIssue { get; set; }
        public int? ClassId { get; set; }
        public int? DepartmentId { get; set; }
        public string? SpecialFunction1 { get; set; }
        public string? SpecialFunction2 { get; set; }
        public string? SpecialFunction3 { get; set; }
        public string? SpecialFunction4 { get; set; }
        public string? SpecialFunction5 { get; set; }
        public string IssueSolution { get; set; } = string.Empty;
        public string Status { get; set; } = string.Empty;
        public string ClosedBy { get; set; } = string.Empty;
        public DateTime? ClosedDate { get; set; }
        public int? CauseId { get; set; }
        public int? SeverityId { get; set; }
        public DateTime? ResponseTime { get; set; }
        public bool? IsFirstCallResolution { get; set; }
        public bool ShouldSuppressEmailForUpdateOperation { get; set; }
        public IEnumerable<UpdateUserDefinedField> UserDefinedFields { get; set; } = Enumerable.Empty<UpdateUserDefinedField>();

    }

    public class UpdateUserDefinedField
    {

        public UpdateUserDefinedField(){}

        public UpdateUserDefinedField(ReadUserDefinedField readUdf)
        {
            if (readUdf == null)
            {
                throw new ArgumentNullException(nameof(readUdf));
            }

            IssueNumber = readUdf.IssueNumber;
            UserDefinedFieldId = readUdf.UserDefinedFieldId;
            Value = readUdf.Value;
        }
        public int IssueNumber { get; set; }
        public int UserDefinedFieldId { get; set; }
        public string Value { get; set; } = string.Empty;

    }

}