﻿using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using UDFUpdater.APIClient;
using UDFUpdater.APIClient.Models;

namespace UDFUpdater
{
    public static class Program
    {
        private const string SiteUrl = "http://localhost/"; //Fill in with the path to issuetrak
        private const string ApiKey = ""; //Fill in with the api key
        private static readonly int[] IssueNumbers = {2, 3}; //list of issue numbers to display

        public static async Task Main()
        {
            var apiClient = new IssuetrakApiClient(SiteUrl, ApiKey);

            var issues = (await apiClient.Issues.List(false, IssueNumbers)).Collection.ToList();
            Ui.DisplayIssueSummary(issues);
            var issueNumber = Ui.GetIssueNumberFromUser();

            while (issueNumber > 0 && issues.Count(i => i.IssueNumber == issueNumber) == 1)
            {
                var issue = issues.Single(i => i.IssueNumber == issueNumber);
                await UpdateIssue(apiClient, issue);

                issues = (await apiClient.Issues.List(false, IssueNumbers)).Collection.ToList();
                Ui.DisplayIssueSummary(issues);
                issueNumber = Ui.GetIssueNumberFromUser();
            }
        }

        private static async Task UpdateIssue(IssuetrakApiClient apiClient, ReadIssue issue)
        {
            var allUdfs = await apiClient.UdfTypes.List();
            Ui.DisplayIssueDetails(allUdfs.Collection, issue);

            var udfId = Ui.GetUdfIdFromUser();
            var issueUdfs = issue.UserDefinedFields.Select(udf => new UpdateUserDefinedField(udf)).ToList();

            while (allUdfs.Collection.Count(udf => udf.UserDefinedFieldId == udfId) == 1)
            {
                var value = Ui.GetUdfValueFromUser();

                UpdateUdf(issue.IssueNumber, issueUdfs, udfId, value);

                udfId = Ui.GetUdfIdFromUser();
            }
            await apiClient.Issues.Update(GetUpdateIssue(issue, issueUdfs));
        }

        private static UpdateIssue GetUpdateIssue(ReadIssue issue, IEnumerable<UpdateUserDefinedField> userDefinedFields)
        {
            return new UpdateIssue(issue, userDefinedFields)
            {
                AssetNumber = null,
                ClassId = null,
                DepartmentId = null,
                IssueSubTypeId = null,
                IssueSubType2Id = null,
                IssueSubType3Id = null,
                IssueSubType4Id = null,
                LocationId = null,
                ProjectId = null,
                SpecialFunction1 = null,
                SpecialFunction2 = null,
                SpecialFunction3 = null,
                SpecialFunction4 = null,
                SpecialFunction5 = null,
                SubStatusId = null
            };
        }

        private static void UpdateUdf(int issueNumber, ICollection<UpdateUserDefinedField> userDefinedFields, int udfId, string udfValue)
        {
            if (userDefinedFields.Any(udf => udf.UserDefinedFieldId == udfId))
            {
                userDefinedFields.Single(udf => udf.UserDefinedFieldId == udfId).Value = udfValue;
            }
            else
            {
                userDefinedFields.Add(new UpdateUserDefinedField
                {
                    IssueNumber = issueNumber,
                    UserDefinedFieldId = udfId,
                    Value = udfValue
                });
            }
        }
    }
}
