﻿using System;
using System.Collections.Generic;
using System.Linq;
using UDFUpdater.APIClient.Endpoints;
using UDFUpdater.APIClient.Models;
using UDFUpdater.Properties;

namespace UDFUpdater
{
    public static class Ui
    {
        public static void DisplayIssueSummary(IEnumerable<ReadIssue> issues)
        {
            if (issues == null) throw new ArgumentNullException(nameof(issues));

            Console.WriteLine();
            foreach (var issue in issues)
            {
                Console.WriteLine(Prompts.IssueSummary, issue.IssueNumber, issue.Subject);
            }
            Console.WriteLine();
        }

        public static void DisplayIssueDetails(IEnumerable<UserDefinedFieldType> allUdfs, ReadIssue issue)
        {
            if (issue == null) throw new ArgumentNullException(nameof(issue));
            if (allUdfs == null) throw new ArgumentNullException(nameof(allUdfs));

            Console.WriteLine();
            Console.WriteLine(Prompts.IssueNumber,issue.IssueNumber);
            Console.WriteLine(Prompts.SubjectLine, issue.Subject);
            Console.WriteLine(Prompts.DividerLine);
            Console.WriteLine(issue.Description);
            Console.WriteLine(Prompts.DividerLine);
            foreach (var udf in allUdfs)
            {
                Console.WriteLine(Prompts.UdfSummary, udf.UserDefinedFieldId, udf.DisplayName, issue.UserDefinedFields.SingleOrDefault(u => u.UserDefinedFieldId == udf.UserDefinedFieldId)?.Value);
            }
            Console.WriteLine();
        }

        public static int GetIssueNumberFromUser()
        {
            Console.Write(Prompts.QueryIssue);
            var userInput = Console.ReadLine();
            return int.TryParse(userInput, out var issueNumber) ? issueNumber : -1;
        }

        public static int GetUdfIdFromUser()
        {
            Console.Write(Prompts.SelectUdf);
            var userInput = Console.ReadLine();
            Console.WriteLine();
            return int.TryParse(userInput, out var udfNumber) ? udfNumber : -1;
        }

        public static string GetUdfValueFromUser()
        {
            Console.Write(Prompts.QueryNewUdfValue);
            return Console.ReadLine();
        }
    }
}
