var http = require("http"); // Used to make http call to API.  use require("https") for http instead of https.
var crypto = require("crypto"); // Used to create HMAC hash required by API.

module.exports =
{
    // Encrypts text using sha512 and base64.
    Encrypt: function(hashKey, messageToBeHashed)
    {
        var hmac = crypto.createHmac("sha512", hashKey);
        hmac.setEncoding("base64");
        hmac.write(messageToBeHashed);
        hmac.end();
        return hmac.read();
    },

    // Creates the headers required by the API.
    MakeHeaders: function(requestId, timestamp, authorizationHash, contentLength)
    {
        var headers =
        {
            "accept": "application/json",
            "content-type": "application/json",
            "content-length": contentLength,
            "X-IssueTrak-API-Request-ID": requestId,
            "X-IssueTrak-API-Timestamp": timestamp,
            "X-IssueTrak-API-Authorization": authorizationHash
        };
        return headers;
    },

    // Calls the API and calls a callback function (onResponse) with the parsed JSON object received from the API call.
    CallAPI: function(httpVerb, server, port, absolutePath, headers, body, onResponse)
    {
        // Make the HTTP call to the API.
        var request = http.request(
            {
                method: httpVerb,
                host: server,
                port: port,
                path: absolutePath,
                headers: headers
            },
            function(response)
            {
                // We've started getting a response from the server.
                console.log("STATUS: " + response.statusCode);
                var result = "";
                response.on("data", function(partialResponse)
                {
                    // We've received a part of the response from the server.
                    result += partialResponse;
                });
                response.on("end", function()
                {
                    // We've received the entire response from the server.
                    if (typeof onResponse === "function")
                    {
                        // Pass it to the caller.
                        onResponse(result);
                    }
                });
                response.on("error", function(e)
                {
                    console.log(e);
                });
            }
        );
        request.write(body);
        request.end();
    }
};
