<?php
$base_url = "http://localhost:56207"; # the base URL for your API installation
$api_key = "test"; # the API Key generated using the API Key Tool
$hash_algorithm = "sha512"; # the hashing algorithm used by the IssueTrak API

################################################################################
# CREATE A NEW NOTE
################################################################################
$request_method = "POST"; # all create operations use "POST" as their HTTP method
$request_path = "/api/v1/notes"; # the path to the note create operation
$request_time_iso8601 = date("c"); # a timestamp in the ISO 8601 format
$request_guid = guid(); # a globally unique identifier, see example implementation below
$request_query = ""; # if query-string parameters were present, they would be recorded here
# The note entity we wish to create ...
$request_body = <<<END
{
  "IssueNumber": 1,
  "CreatedDate": "2014-08-25T12:32:32-04:00",
  "CreatedBy": "APIUser",
  "ShouldSuppressEmailForCreateOperation": false,
  "NoteText": "Testing note creation from PHP.",
  "IsPrivate": false,
  "IsRichText": false
}
END;

# gather the request information into a summary;
# note that the order of the elements in this array and the strtolower() operation on the GUID and path are mandatory
$request_summary = implode("\n" , [$request_method, strtolower($request_guid), $request_time_iso8601, strtolower($request_path), $request_query, $request_body]);
# use the summary to generate a signature for authentication of this request
$request_signature = base64_encode(hash_hmac($hash_algorithm, $request_summary, $api_key, true));

# assemble HTTP headers for this request; note that the headers below are mandatory
$http_headers = array(
    "Content-Type: text/json",
    "X-IssueTrak-API-Timestamp: " . $request_time_iso8601,
    "X-IssueTrak-API-Request-ID: " . $request_guid,
    "X-IssueTrak-API-Authorization: " . $request_signature
);

$curl = curl_init(); # initialize curl
curl_setopt($curl, CURLOPT_POST, 1); # this request will be a POST
curl_setopt($curl, CURLOPT_URL, $base_url . $request_path . $request_query); # tell curl about the request URL
curl_setopt($curl, CURLOPT_POSTFIELDS, $request_body); # tell curl what the body content will be
curl_setopt($curl, CURLOPT_HTTPHEADER, $http_headers); # tell curl about our array of headers
curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1); # tell curl that we'd like the result returned normally

$new_note_id = curl_exec($curl); # make the request
echo "The New NoteID: " . $new_note_id . "\n\n"; # the response should be a newly created NoteID

################################################################################
# RETRIEVE THE NOTE WE JUST CREATED
################################################################################
$request_method = "GET"; # all retrieval operations use "GET" as their HTTP method
$request_path = "/api/v1/notes/" . $new_note_id; # the path to the note we just created
$request_time_iso8601 = date("c"); # a timestamp in the ISO 8601 format
$request_guid = guid(); # a globally unique identifier, see example implmentation below
$request_query = ""; # if query-string parameters were present, they would be recorded here
$request_body = ""; # no additional information is required to retrieve the Note, but we still need this place-holder

# gather the request information into a summary;
# note that the order of the elements in this array and the strtolower() operation on the path are mandatory
$request_summary = implode("\n" , [$request_method, $request_time_iso8601, strtolower($request_path), $request_query, $request_body]);
# use the summary to generate a signature for authentication of this request
$request_signature = base64_encode(hash_hmac($hash_algorithm, $request_summary, $api_key, true));

# assemble HTTP headers for this request; note that the headers below are mandatory
$http_headers = array(
    "Content-Type: text/json",
    "X-IssueTrak-API-Timestamp: " . $request_time_iso8601,
    "X-IssueTrak-API-Request-ID: " . $request_guid,
    "X-IssueTrak-API-Authorization: " . $request_signature
);

$curl = curl_init(); # initialize curl
curl_setopt($curl, CURLOPT_GET, 1); # this request will be a GET
curl_setopt($curl, CURLOPT_URL, $base_url . $request_path . $request_query); # tell curl about the request URL
curl_setopt($curl, CURLOPT_HTTPHEADER, $http_headers); # tell curl about our array of headers
curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1); # tell curl that we'd like the result returned normally

$note_record = curl_exec($curl); # make the request
echo "The New Note: " . $note_record . "\n\n"; # the response should be the Note we created above

################################################################################
# RETRIEVE SEVERAL ISSUES WITHOUT NOTES
################################################################################
$request_method = "GET"; # all retrieval operations use "GET" as their HTTP method
$request_path = "/api/v1/issues/false/list"; # the path for Issue list retrieval without notes
$request_time_iso8601 = date("c"); # a timestamp in the ISO 8601 format
$request_guid = guid(); # a globally unique identifier, see example implmentation below
$request_query = "?issueNumbers=1,2,3"; # issueNumbers is a query-string parameter
$request_body = ""; # no additional information is required to retrieve the Issues, but we still need this place-holder

# gather the request information into a summary;
# note that the order of the elements in this array and the strtolower() operation on the path are mandatory
$request_summary = implode("\n" , [$request_method, $request_time_iso8601, strtolower($request_path), $request_query, $request_body]);
# use the summary to generate a signature for authentication of this request
$request_signature = base64_encode(hash_hmac($hash_algorithm, $request_summary, $api_key, true));

# assemble HTTP headers for this request; note that the headers below are mandatory
$http_headers = array(
    "Content-Type: text/json",
    "X-IssueTrak-API-Timestamp: " . $request_time_iso8601,
    "X-IssueTrak-API-Request-ID: " . $request_guid,
    "X-IssueTrak-API-Authorization: " . $request_signature
);

$curl = curl_init(); # initialize curl
curl_setopt($curl, CURLOPT_GET, 1); # this request will be a GET
curl_setopt($curl, CURLOPT_URL, $base_url . $request_path . $request_query); # tell curl about the request URL
curl_setopt($curl, CURLOPT_HTTPHEADER, $http_headers); # tell curl about our array of headers
curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1); # tell curl that we'd like the result returned normally

$issues_collection = curl_exec($curl); # make the request
echo "A Collection of Issues: " . $issues_collection . "\n\n"; # the response should contain Issues 1, 2, and 3

################################################################################
# END OF EXECUTION
################################################################################
echo "Press any key to exit ...";
fread(STDIN, 1); # blocks until user presses any key

################################################################################
# EXAMPLE GUID GENERATOR
################################################################################
function guid()
{
    if (function_exists('com_create_guid'))
    { # this happens on Windows
        return com_create_guid();
    }
    else
    { # this happens on other platforms
        $charid = md5(uniqid(rand(), true));
        $hyphen = chr(45); # "-"
        $uuid = chr(123) # "{"
              . substr($charid, 0, 8) . $hyphen
              . substr($charid, 8, 4) . $hyphen
              . substr($charid,12, 4) . $hyphen
              . substr($charid,16, 4) . $hyphen
              . substr($charid,20,12)
              . chr(125); # "}"
        return $uuid;
    }
}
?>
